library(stringr)
library(Biostrings)

# Function call_BOriS=======
# Main function of BOriS, which is a tool that is designed to identify origin of replication (oriC) sequences in bacterial chromosomes. In contrast to other tools, BOriS recognizes
# oriC sequences based on their motif composition and is thus not restricted to full chromosomes as an input. For more information and background, see ...
# 
# 
#
# Input variables:
# path_to_data -- A string that contains the path to the FASTA file which will be searched for oriC sequences. This file can contain one or more sequences.
# path_to_seeds -- A string that contains the path to the FASTA file containing the seed sequences (provided with this script)
# path_to_lsgkmpredict -- A string that contains the path to gkmpredict, the predict function of LS-GKM (provided with this script)
# path_to_model -- A string that contains the path to the LS-GKM model used in BOriS (provided with this script)
# path_to_output_result -- A string that contains the path to the folder to which the output of BOriS will be printed
# fragmentsize -- An integer defining the target length of the fragments in nucleotides (default: 1250)
# linear -- A boolean specifying whether the input sequence(s) are linear (or circular, if FALSE, default: FALSE)
# cores -- An integer declaring the number of cores to be used by gkmpredict (can assume values of 1, 4 or 16, default: 4)
# single_chromosome -- A boolean indicating whether the fragments in full_dataframe belong to a single chromosome (as e.G. wuth contigs) or different identifier represent 
#                   different chromosomes (default: TRUE)
#
# Output:
# fragment_dataframe -- A dataframe containing information regarding the fragments created and evaluated earlier. Columns: 
#    seed -- The seed sequence that was used when creating the current fragment
#    identifier -- The identifier of the sequence this fragment was taken from
#    begin -- The start position of the fragment relative to the sequence this fragment was taken from
#    end -- The end position of the fragment relative to the sequence this fragment was taken from
#    sequencelength -- The length of the sequence this fragment was taken from
#    fragment -- The sequence of the fragment
# 
call_BOriS <- function(path_to_data, path_to_seeds, path_to_lsgkmpredict, path_to_model, path_to_output_result,
                       fragmentsize = 1250, linear = FALSE, cores = 4, single_chromosome = TRUE){
  ## Read both the seed sequences and the input file and format the sequence names of the latter
  #print <- function(sometext){
  #  shinyoutput$borismsg <- renderText({sometext}) 
  #}
  
  
  print("Started BOriS...")
  data <- readDNAStringSet(path_to_data)  
  names(data) <- unname(sapply(names(data), function(x) strsplit(x, " ")[[1]][1]))
  seeds <- readDNAStringSet(path_to_seeds) 
  
  ## Create sequence fragments around the seed sequences that will serve as input for LSGKM. This needs to be done in a loop because a FASTA file might contain more than one sequence
  print("Creating fragments")
  pos_fragments <- create_and_eval_fragments(data, seeds, path_to_output_result, path_to_lsgkmpredict, path_to_model, fragmentsize, linear, cores)
  print("Done calling LS-GKM...")
  pos_fragments["orientation"] = "+"
 
  # get the fragments to predict their possibilies to be an Ori by LS-GKM
  print("Creating reverse complement fragments")
  rv_data <- reverseComplement(data)
  rev_fragments <- create_and_eval_fragments(rv_data, seeds, path_to_output_result, path_to_lsgkmpredict, path_to_model, fragmentsize, linear, cores)
  print("Done calling LS-GKM...")
  rev_fragments["orientation"] = "-"
  
  if ((length(pos_fragments[, 1]) == 0) && (length(rev_fragments[, 1]) == 0)){
    print("No fragments identified.")
    result_is_empty = T
  }else if(length(pos_fragments[, 1]) == 0){
    print("No fragments identified in + orientation.")
    full_dataframe <- rev_fragments
    result_is_empty = F
  }else if (length(rev_fragments[, 1]) == 0){
    print("No fragments identified in - orientation.")
    full_dataframe <- pos_fragments
    result_is_empty = F
  }else{
    full_dataframe <- rbind(pos_fragments, rev_fragments)
    result_is_empty = F
  }
  
  if (result_is_empty){
    print("Nothing to print...")
  }else{
    print("Normalize and Select fragments from LS-GKM output...")
    selected_fragments <- select_fragments_with_rejection(full_dataframe, single_chromosome, 0.41, 0.99)
    
    print("Printing results to files...") 
    create_output(selected_fragments,
                  paste(path_to_output_result, paste("BOriS_", strsplit(strsplit(path_to_data, "/")[[1]]
                                                                        [length(strsplit(path_to_data, "/")[[1]])], "\\.fna")[[1]][1], "_fa.fasta", sep=""), sep = "/"),
                  paste(path_to_output_result, paste("BOriS_", strsplit(strsplit(path_to_data, "/")[[1]]
                                                                        [length(strsplit(path_to_data, "/")[[1]])], "\\.fna")[[1]][1], "_df.csv", sep=""), sep = "/"),
                  paste(path_to_output_result, paste("BOriS_", strsplit(strsplit(path_to_data, "/")[[1]]
                                                                        [length(strsplit(path_to_data, "/")[[1]])], "\\.fna")[[1]][1], "_ambiguous_df.csv", sep=""), sep = "/")
    )
  }
  print("Done!")
}


# Function search_for_seeds=======
# Identifies the positions of occurences of a list of seed sequences in a DNA sequence
#
# This function creates a list of fragments from one DNA sequence in data based on the positon of the seed sequences stored in seeds. It determines the positions of the seed sequences by calling
# search_for_seeds() and stores them in fragment_dataframe. To this positions, fragments of the size n are created using different methods depending whether the input sequence is linear or circular.
#
# Input variables:
# data -- A DNAStringSet object containing a single DNA sequence
# seeds -- A DNAStringSet object containing a set of seed sequences i.e. short DNA sequences that will serve as anchors for the fragments
# fragmentlength -- An integer defining the target length of the fragments in nucleotides
#
# Output:
# seed_dataframe -- A dataframe containing information regarding the seed positions. Columns: 
#    seed -- The seed sequence that was used when creating the current fragment
#    identifier -- The identifier of the sequence this fragment was taken from
#    begin -- The start position of the fragment relative to the sequence this fragment was taken from
#    end -- The end position of the fragment relative to the sequence this fragment was taken from
#    sequencelength -- The length of the sequence this fragment was taken from
#    fragment -- Column is set to FALSE and acts as a place holder
# 
search_for_seeds <- function(data, seeds, fragmentlength){
  X <- c()
  for(s in 1:length(seeds)){ # for each seed in our seedlist the positions in data are determined
      pattern <- names(seeds)[s]
      data_as_string <- toString(data)
      sequencelength <- width(data)
      # if the seed is present on the DNA sequence in data
      if(str_detect(data_as_string, pattern)){
        # we locate the seed sequence and save the positions in the result vector 
        # as well as we're saving the position of the fragment that will be created later
        matching_matrix<- as.matrix(as.matrix(str_locate_all(data_as_string,pattern))[[1]])
        for(i in 1:length(matching_matrix[,1])){
          begin <- matching_matrix[[i,1]] - ((fragmentlength-9)/2+0.5) #extending the beginning of the seed sequence to the fragment start position
          if (begin < 0){
            begin <- sequencelength + begin
          }
          end <-matching_matrix[[i,2]] + ((fragmentlength-9)/2+0.5) #extending the end of the seed sequence to the fragment end position
          if (end > sequencelength){
            end <- end - sequencelength     
          }
          seqname <- names(data)[1] 
          X <- rbind(X, c(pattern, seqname, begin, end, sequencelength))
        }
      }
    
  }
  # changing the list into a data.frame
  seed <- as.vector(X[,1])
  begin <- as.vector(X[,3]) 
  end <-as.vector(X[,4])
  sequencelength <- as.vector(X[,5])
  identifier <- as.vector(X[,2]) 
  fragment <- vector(mode="logical",length=length(seed))
  raw_output <- vector(mode="logical",length=length(seed))
  seed_dataframe <- data.frame(seed, identifier, begin, end, sequencelength, fragment, raw_output)

  return(seed_dataframe)
}

# Function create_fragments=======
# Extracts sequence fragments from a DNA sequence based on a list of seed sequences
#
# This function creates a list of fragments from one DNA sequence in data based on the positon of the seed sequences stored in seeds. It determines the positions of the seed sequences by calling
# search_for_seeds() and stores them in fragment_dataframe. To this positions, fragments of the size n are created using different methods depending whether the input sequence is linear or circular.
#
# Input variables:
# data -- A DNAStringSet object containing a single DNA sequence
# seeds -- A DNAStringSet object containing a set of seed sequences i.e. short DNA sequences that will serve as anchors for the fragments
# n -- An integer defining the target length of the fragments in nucleotides
# linear -- A boolean specifying whether the input sequence(s) are linear (or circular, if FALSE)
#
# Output:
# fragment_dataframe -- A dataframe containing information regarding the fragments created and evaluated earlier. Columns: 
#    seed -- The seed sequence that was used when creating the current fragment
#    identifier -- The identifier of the sequence this fragment was taken from
#    begin -- The start position of the fragment relative to the sequence this fragment was taken from
#    end -- The end position of the fragment relative to the sequence this fragment was taken from
#    sequencelength -- The length of the sequence this fragment was taken from
#    fragment -- The sequence of the fragment
# 
create_fragments <- function(data, seeds, n, linear){
  genome_sequence <- toString(data)
  genome_length <- width(data)
  
  # call search_for_seeds to generate fragment_dataframe with the positions of the seed sequences
  fragment_dataframe <- search_for_seeds(data, seeds, n)
  
  if (length(fragment_dataframe[, 1]) != 0){
    # for each seed in the dataframe we'll generate a fragment, if possible
    for(i in 1:nrow(fragment_dataframe)){   
      fragment_dataframe_row <- as.matrix(as.matrix(fragment_dataframe)[i,])
      begin <-  as.numeric(fragment_dataframe_row[3])
      end <-  as.numeric(fragment_dataframe_row[4])
      
      # CASE 1: GENOME IS LINEAR
      if(linear){
        # CASE_1.1: Fragment does not cross the sequence borders 
        if((begin >= 0) && (end <= genome_length)){ 
          fragment <-substr(genome_sequence, begin, end)
          #Check if fragment contains other characters than A,C,G,T; if yes -> no fragment
          if(!(grepl("[^ACGT]",fragment))){fragment_dataframe[i,6]<- fragment}
        }
        # CASE_1.2: Fragment exceeds the sequence limits: Not possible -> no fragment
        
      # CASE 2: GENOME IS CIRCULAR
      }else{
        if((begin<0)&&(end<=genome_length)){ ## CASE_2.1: Fragment overlaps the left side of the DNA sequence 
          right_fragment <- substr(genome_sequence,end+1,end)
          seed <- substr(genome_sequence,begin,end)
          left_fragment <- substr(genome_sequence,1,begin-1)
          overlay <- abs(width(left_fragment))
          overlay_left_fragment <- substr(genome_sequence,width(genome_sequence)-(overlay-1),width(genome_sequence))
          fragment <-paste(overlay_left_fragment,left_fragment,seed,right_fragment,sep="")
          if(!(grepl("[^ACGT]",fragment))){fragment_dataframe[i,6]<- fragment}
          
        }else if((begin>=0)&&(end>genome_length)){ ## CASE_2.2: Fragment overlaps the right side of the DNA sequence
          left_fragment <- substr(genome_sequence,begin,begin-1)
          seed <- substr(genome_sequence,begin,end)
          right_fragment <- substr(genome_sequence,end+1,width(genome_sequence))
          overlay <- abs(width(right_fragment))
          overlay_right_fragment <- substr(genome_sequence,1,(overlay)) 
          fragment <-paste(left_fragment,seed,right_fragment,overlay_right_fragment,sep="")
          if(!(grepl("[^ACGT]",fragment))){fragment_dataframe[i,6]<- fragment}
          
        }else if((begin>=0)&&(end<=genome_length)){ ## CASE_2.3: Fragment is enclosed by the DNA sequence
          fragment <-substr(genome_sequence, begin, end)
          if(!(grepl("[^ACGT]",fragment))){fragment_dataframe[i,6]<- fragment}
        }
      }
    }
    
    ##Remove all rows in the dataframe which do not contain a fragment - for which fragment creation failed for any reason
    remove_rows = which(fragment_dataframe[["fragment"]] == "FALSE")
    if (length(remove_rows) != 0){
      fragment_dataframe = fragment_dataframe[-remove_rows, ]
    }
  }
  
  return(fragment_dataframe)
}


# Function write_file_for_LSGKM=======
# Writes the information on the fragments stored in fragment_dataframe to a FASTA formatted file
#
# This is necessary because LS-GKM wil be called via the command line and thus information cannot be handed over directly
#
# Input variables:
# fragment_dataframe -- A dataframe containing information regarding the fragments created and evaluated earlier
# path_to_fragmentfile -- A string containing the absolute path to the file that will be created
# 
#
# Output:
# None
# 
write_file_for_LSGKM <- function(fragment_dataframe, path_to_fragmentfile){
  for (i in 1:nrow(fragment_dataframe)){
    fragment_dataframe_row<- as.matrix(as.matrix(fragment_dataframe)[i,])
    name <- unname(fragment_dataframe_row["identifier", ])
    fragment <-  unname(fragment_dataframe_row["fragment", ])
    write(paste(">",i,"/", name, "\n", fragment, sep=""), file=path_to_fragmentfile, append = T)
  }
}



# Function create_and_eval_fragments=======
# Extracts sequence fragments from the input sequence based on a list of seed sequences and evaluates them using a LS-GKM model. 
#
# This function loops through the sequences in data, calls the function create_fragments() with each on them and collects the outputs in fragment_dataframe. This is 
# written to disk using the function write_file_for_LSGKM() because gkmpredict only accepts files as input. Using the function predict_fragments(), gkmpredict (which 
# is a part of LS-GKM) is called to evaluate (i.e. used as input for the predict function of the classifier) the fragments generated earlier. Finally, all temporary files 
# are removed.
#
# Input variables:
# data -- A DNAStringSet object as is produced by readDNAStringSet() from the Biostrings package containging one or multiple DNA sequences
# path_to_output_result -- A string containing the absolute path to the folder in which temp files will be created
# path_to_lsgkmpredict -- A string containing the absolute path to gkmpredict from the LS-GKM package
# path_to_model -- A string containing the absolute path to a trained LS-GKM model
# fragmentsize -- An integer defining the length of the created fragments in nukleotides
# linear -- A boolean specifying whether the input sequence(s) are linear (or circular, if FALSE)
# cores -- An integer declaring the number of cores to be used by gkmpredict (can assume values of 1, 4 or 16)
#
# Output:
# fragment_dataframe -- A dataframe containing information regarding the fragments created and evaluated earlier. Columns: 
#    seed -- The seed sequence that was used when creating the current fragment
#    identifier -- The identifier of the sequence this fragment was taken from
#    begin -- The start position of the fragment relative to the sequence this fragment was taken from
#    end -- The end position of the fragment relative to the sequence this fragment was taken from
#    sequencelength -- The length of the sequence this fragment was taken from
#    fragment -- The sequence of the fragment
#    raw_output -- The classification score that was calculated for the fragment by gkmpredict using the model in path_to_model
# 
create_and_eval_fragments <- function(data, seeds, path_to_output_result, path_to_lsgkmpredict, path_to_model, fragmentsize, linear, cores){
  fragment_path = paste(path_to_output_result, "fragments_", as.numeric(Sys.time())*1000, ".fa", sep = "")
  for (counter in c(1:length(data))){
    if (counter == 1){
      fragment_dataframe <- create_fragments(data[counter], seeds, fragmentsize, linear)
    }else{
      tmp <- create_fragments(data[counter], seeds, fragmentsize, linear)
      fragment_dataframe <- rbind(fragment_dataframe, tmp)
    }
  }
  if (length(fragment_dataframe[, 1]) == 0){
    write_file_for_LSGKM(fragment_dataframe, fragment_path)
    print("Calling LS-GKM...")
    tmp_path = paste(path_to_output_result, "tmp_", as.numeric(Sys.time())*1000, ".fa", sep = "")
    fragment_dataframe <- predict_fragments(fragment_dataframe, path_to_lsgkmpredict, path_to_model, fragment_path, tmp_path,
                                            fragmentsize = fragmentsize, linear = linear, cores = cores)
    invisible(file.remove(tmp_path))
    invisible(file.remove(fragment_path))
  }
  return(fragment_dataframe)
}



# Function predict_fragments=======
# Calls gkmpredict on the fragments in fragment_dataframe_input and path_to_fragmentdataframe and adds this information to fragment_dataframe_input
#
# 
#
# Input variables:
# fragment_dataframe_input --  A dataframe containing information regarding the fragments created on the basis of the seed sequences earlier
# path_to_lsgkmpredict -- A string containing the absolute path to gkmpredict from the LS-GKM package
# path_to_model -- A string containing the absolute path to a trained LS-GKM model
# path_to_fragmentdataframe -- A string containing the path to the FASTA file created by write_file_for_LSGKM
# path_to_output_result -- A string containing the path where the output of LS-GKM gkmpredict will be saved to and read from
# fragmentsize -- An integer defining the length of the created fragments in nucleotides
# linear -- A boolean specifying whether the input sequence(s) are linear (or circular, if FALSE)
# cores -- An integer declaring the number of cores to be used by gkmpredict (can assume values of 1, 4 or 16)
#
# Output:
# fragment_dataframe_input -- A dataframe containing information regarding the fragments created and evaluated earlier. Columns: 
#    seed -- The seed sequence that was used when creating the current fragment
#    identifier -- The identifier of the sequence this fragment was taken from
#    begin -- The start position of the fragment relative to the sequence this fragment was taken from
#    end -- The end position of the fragment relative to the sequence this fragment was taken from
#    sequencelength -- The length of the sequence this fragment was taken from
#    fragment -- The sequence of the fragment
#    raw_output -- The classification score that was calculated for the fragment by gkmpredict using the model in path_to_model
# 
# Input:
# List of fragments from create_fragments()
# saves the list of fragments into a FASTAfile and Will predict the propabilitly of the fragment being an ori
# by using lsgkm (N1250_L10_K6_D4_T4).
# Reads out the predvalues pred by lsgkm method writes them into a File 
# Output:
# fragment_dataframe - List of framents and their predictionvalue
#
#
predict_fragments <- function(fragment_dataframe_input, path_to_lsgkmpredict, path_to_model, path_to_fragmentdataframe, path_to_output_result, 
                              fragmentsize = 1250, linear = TRUE, cores = 4){
  lsgkm_command =  paste(path_to_lsgkmpredict, path_to_fragmentdataframe, path_to_model, path_to_output_result, "-T", cores, sep = " ")
  system(lsgkm_command) ##TODO replace system() by system2()!!
  fragmentpredictionlist <- read.csv(path_to_output_result, sep ="\t", header = F)
  fragment_dataframe_input[["raw_output"]] <- fragmentpredictionlist[, 2]
  return(fragment_dataframe_input)
}

# Function apply_cutoffs=======
# Select those fragments from fragment_dataframe that are regarded to contain oriC sequences and those fragments for which BOriS will withhold judgement.
#
# This function first normalizes the classifation scores present in a subset of full_dataframe using the function normalize and then employs two different cutoffs that define a 
# range in which fragments are regarded positive (1>=x>=upper_cutoff), rejected (i.e. a classification decision is withhold; upper_cutoff>x>lower_cutoff) or negative
# (x<=lower_cutoff), where x is the value of fragment_dataframe[["normalized"]].
#
# Input variables:
# fragment_dataframe -- A dataframe containing information regarding all fragments gathered from the input FASTA file, in both normal and reverse complement direction 
#                   with classification score as determined by LS-GKM.
# lower_cutoff -- A float determining which fragments are regarded negative (i.e. non-oriC) and rejected (i.e. a decision is withhold)
# upper_cutoff -- A float determining which fragments are regarded rejected (i.e. a decision is withhold) and positive (i.e. containing oriC)
#
# Output:
#    A list of two dataframes containing selected fragments. selected_fragments$positive contains sequences that are identified as oriC sequences, 
#                       selected_fragments$rejected contains those fragments for which no clear decision is made. Columns:
#    seed -- The seed sequence that was used when creating the current fragment
#    identifier -- The identifier of the sequence this fragment was taken from
#    begin -- The start position of the fragment relative to the sequence this fragment was taken from
#    end -- The end position of the fragment relative to the sequence this fragment was taken from
#    sequencelength -- The length of the sequence this fragment was taken from
#    fragment -- The sequence of the fragment
#    raw_output -- The classification score that was calculated for the fragment by gkmpredict using the model in path_to_model
#    orientation -- The orientation of the fragment, either "+" or "-"
#    normalized -- The classification score after normalization as performed by normalize
#
apply_cutoffs <- function(fragment_dataframe, lower.cutoff = 0.41, upper.cutoff = 0.99){
  rejected_dataframes <- fragment_dataframe[which(fragment_dataframe[["normalized"]] > lower.cutoff & fragment_dataframe[["normalized"]] < upper.cutoff), ]
  positive_dataframes <- fragment_dataframe[which(fragment_dataframe[["normalized"]] >= upper.cutoff), ]
  return(list("positive" = positive_dataframes, "rejected" = rejected_dataframes) )
}



# Function normalize=======
# Normalizes the classification score of the fragments in fragment_dataframe to a range of [0, 1]
#
# The values in fragment_dataframe[["raw_output"]] are normalized to [0, 1] by either (if min(fragment_dataframe[["raw_output"]]) < 0) adding 
# abs(min(fragment_dataframe[["raw_output"]])) to all values or (if min(fragment_dataframe[["raw_output"]]) > 0) subtracting abs(min(fragment_dataframe[["raw_output"]])) 
# from all values and then dividing all values by the new maximum value. The resulting values are stored to fragment_dataframe[["normalized"]].
#
# Input variables:
# fragment_dataframe -- A dataframe containing information regarding all fragments gathered from the input FASTA file, in both normal and reverse complement direction 
#                   with classification score as determined by LS-GKM..
#
# Output:
# fragment_dataframe -- The input dataframe with the added "normalized" column. 
#
normalize<-function(fragment_dataframe){
  abs_min <-abs(min(fragment_dataframe[["raw_output"]], na.rm = T))
  for(i in 1:nrow(fragment_dataframe)){
    fragment_dataframe[i, "normalized"]<- (fragment_dataframe[i,  "raw_output"]+abs_min)
  }
  
  abs_max <- abs(max(fragment_dataframe[["normalized"]], na.rm = T))
  for(i in 1:nrow(fragment_dataframe)){
    fragment_dataframe[i, "normalized"]<- fragment_dataframe[i, "normalized"]/abs_max
  }
  return(fragment_dataframe)
}


# Function select_fragments_with_rejection=======
# Select those fragments from full_dataframe that are regarded to contain oriC sequences and those fragments for which BOriS will withhold judgement.
#
# This function first normalizes the classifation scores present in full_dataframe using the function normalize() and then employs two different cutoffs that define a 
# range in which fragments are regarded positive (1>=x>=upper_cutoff), rejected (i.e. a classification decision is withhold; upper_cutoff>x>lower_cutoff) or negative
# (x<=lower_cutoff) by using the function apply_cutoffs(). This function is able to handle files that contain sequences from a single chromosome and files that contain 
# sequences from multiple chromosomes differently (the latter is necessary e.G. when applying BOriS to metagenomic data). 
#
# Input variables:
# full_dataframe -- A dataframe containing information regarding all fragments gathered from the input FASTA file, in both normal and reverse complement direction 
#                   with classification score as determined by LS-GKM..
# single_chromosome -- A boolean indicating whether the fragments in full_dataframe belong to a single chromosome (as e.G. wuth contigs) or different identifier represent 
#                   different chromosomes
# lower_cutoff -- A float determining which fragments are regarded negative (i.e. non-oriC) and rejected (i.e. a decision is withhold)
# upper_cutoff -- A float determining which fragments are regarded rejected (i.e. a decision is withhold) and positive (i.e. containing oriC)
#
# Output:
# selected_fragments -- A list of two dataframes containing selected fragments. selected_fragments$positive contains sequences that are identified as oriC sequences, 
#                       selected_fragments$rejected contains those fragments for which no clear decision is made. Columns:
#    seed -- The seed sequence that was used when creating the current fragment
#    identifier -- The identifier of the sequence this fragment was taken from
#    begin -- The start position of the fragment relative to the sequence this fragment was taken from
#    end -- The end position of the fragment relative to the sequence this fragment was taken from
#    sequencelength -- The length of the sequence this fragment was taken from
#    fragment -- The sequence of the fragment
#    raw_output -- The classification score that was calculated for the fragment by gkmpredict using the model in path_to_model
#    orientation -- The orientation of the fragment, either "+" or "-"
#    normalized -- The classification score after normalization as performed by normalize
#
select_fragments_with_rejection <- function(full_dataframe, single_chromosome, lower_cutoff, upper_cutoff){
  if (single_chromosome){ #CASE 1: All sequences in full_dataframe belong to a single chromosome
    full_dataframe <- normalize(full_dataframe)
    selected_fragments <- apply_cutoffs(full_dataframe, lower_cutoff, upper_cutoff)
  }else{ #CASE 2: Every identifier in full_dataframe represents a separate chromosome
    chromosome_ids <- unique(as.character(full_dataframe[["identifier"]]))
    for (counter in c(1:length(chromosome_ids))){
      identifier_fragments <- full_dataframe[which(full_dataframe[["identifier"]] == chromosome_ids[counter]), ]
      identifier_fragments <- normalize(identifier_fragments)
      if (counter == 1){
        selected_fragments <- apply_cutoffs(identifier_fragments, lower_cutoff, upper_cutoff)
      }else{
        selected_fragments_identifier <- apply_cutoffs(identifier_fragments, lower_cutoff, upper_cutoff)
        positives_tmp <- rbind(selected_fragments$positive, selected_fragments_identifier$positive)
        rejected_tmp <- rbind(selected_fragments$rejected, selected_fragments_identifier$rejected)
        selected_fragments <- list("positive"=positives_tmp, "rejected"=rejected_tmp)
      }
    }
  }
  return(selected_fragments)
}




# Function create_output=======
# Creates  files in both FASTA and csv format from selected_fragment_dataframe
#
# For different use cases, different files are created here. The positive sequences are stored totarget_fasta as a FASTA file for use in regular DNA pipelines. Additionally, the 
# dataframe selected_fragment_dataframe$positive is written to target_dataframe in the csv format. Similarly, the rejected rows that are present in 
# selected_fragment_dataframe$rejected are written to target_rejected_dataframe in a csv format, but not, additionally, stored in the FASTA format.
#
# Input variables:
# selected_fragment_dataframe -- A list of two dataframes containing selected fragments. selected_fragments$positive contains sequences that are identified as oriC sequences, 
#                       selected_fragments$rejected contains those fragments for which no clear decision is made -- as returned by select_fragments_with_rejection.
# target_fasta -- A string indicating the position to which a FASTA file containing the positive oriC fragments is written to
# target_dataframe -- A string indicating the position to which a csv file containing the dataframe of positive oriC fragments is written to
# target_rejected_dataframe --  A string indicating the position to which a csv file containing the dataframe of rejected oriC fragments is written to
#
# Output:
# None
# 
create_output <- function(selected_fragment_dataframe, target_fasta, target_dataframe, target_rejected_dataframe){
  write.csv(selected_fragment_dataframe$positive, target_dataframe)
  write.csv(selected_fragment_dataframe$rejected, target_rejected_dataframe)
  
  tmp <- apply(selected_fragment_dataframe$positive, 1, function(x) write(paste(">", x[["identifier"]], " ", x[["begin"]], ":", x[["end"]], 
                      " ", x[["orientation"]], " ", x[["raw_output"]], " ", x[["normalized"]], "\n", x[["fragment"]], sep=""), file = target_fasta, append = T))
}



#####
## call BOriS from the command line ====
# If called from the command line, this script takes two arguments: The path to the file that will serve as input as well as the path to the folder in which the output of 
# BOriS will be saved

args = commandArgs(trailingOnly=TRUE) s
infile = args[1]
target = args[2]
source_folder = args[3]
 
if (substr(target, nchar(target), nchar(target)) != "/"){
 target = paste(target, "/", sep = "")
}
 
call_BOriS(path_to_data = infile,
          path_to_seeds = paste(source_folder, "seeds.fa", sep = ""),
          path_to_lsgkmpredict = paste(source_folder, "gkmpredict", sep = ""),
          path_to_model = paste(source_folder, "ls-gkm_train_model_1250_L10_K6_D4_T4.model.txt", sep = ""),
          path_to_output_result = target,
          1250, FALSE, single_chromosome = TRUE)
 